// ========================================================

// ========================================================

      
    /**********************************************************
//		  MD1239		  N79E814-TSSOP20
//		  |-------------------|
//                |            nRESET |--1-- (P3.1)
//                |              DIO0 |--2-- (P3.0,NIRQ)
//                |              DIO1 |--3--
//                |              DIO2 |--4--
//                |               GND |--5--  GND
//                |               VDD |--6--  VCC
//                |              DIO5 |--7-- 
//                |               SCK |--8-- (P1.4,SCK)
//                |              MISO |--9-- (P1.3,SDO)
//                |              MOSI |--10- (P1.2,SDI)
//                |               NSS |--11- (P1.1,NSEL)
//                |            LNA_EN |--12- (P1.0,LNA)
//                | ------------------|  
//      

//		MD002


//  RF module:           MD1239 (Receiver only)
//  Carry Frequency:     315MHz/434MHz/868MHz/915MHz
//  Bit Rate:            1.2Kbps/2.4Kbps/4.8Kbps/9.6Kbps
//  Receive Bandwidth:   83KHz
//  Coding:              NRZ
//  Packet Format:       
//  Rx Current:          about 16mA  (typ.)                 
/**********************************************************/
//void MD1239_Config(void);
//void MD1239_EntryRx(void);
//void MD1239_ClearFIFO(void);
//void MD1239_Sleep(void);
//void MD1239_Standby(void);
////u8 MD1239_ReadRSSI(void);
//unsigned char MD1239_RxPacket(void);
//void MD1239_EntryTestRx(void);
//void MD1239_TestRx(void);

/**********************************************************
**Parameter table define
**********************************************************/

//RX	
code unsigned int MD1239FreqTbl[4][3] = 
{ 
  {0x074e, 0x08c0, 0x0900}, //315MHz
  {0x076c, 0x0880, 0x0900}, //434MHz
  {0x07d9, 0x0800, 0x0900}, //868MHz  <---
  {0x07e4, 0x08c0, 0x0900}, //915MHz
};

code unsigned int MD1239RateTbl[4][2] = 
{
  {0x03d0, 0x0455},         //BR=0.6K BW=83.333K  <--- for test mode
  //{0x0368, 0x042B},         //BR=1.2K BW=83.333K
  {0x0334, 0x0415},         //BR=2.4K BW=83.333K  <---
  //{0x031A, 0x040B},         //BR=4.8K BW=83.333K
  {0x030D, 0x0405},         //BR=9.6K BW=83.333K
  {0x0303, 0x0441},		    //BR=38.4K   off mode high bitrate <--
};


code unsigned int MD1239ConfigTbl[18] = 
{ 
  0x0200,                   //RegDataModul, FSK Packet  
  0x0502,                   //RegFdevMsb, 241*61Hz = 35KHz  
  0x0641,                   //RegFdevLsb
  //0x0500,		   //RegFdevMsb,0x6a(106)*61.0351563Hz=6.4697kHz
  //0x066a,					//RegFdevLsb
  0x130F,                   //RegOcp, Disable OCP
  0x1952,                   //RegRxBw , RxBW, 83KHz
  //0x1944,					//RegRxBw, RxBW=31.25kHz
  
  0x2C00,                   //RegPreambleMsb  
  0x2D05,                   //RegPreambleLsb, 5Byte Preamble
  0x2E90,                   //enable Sync.Word, 2+1=3bytes
  0x2FAA,                   //0xAA, SyncWord = aa2dd4
  0x3056,//26,//2D,                   //0x2D
  0x3178,//96,//D4,                   //0xD4  
  0x3700,                   //RegPacketConfig1, Disable CRCNRZ encode
  0x3806,//15,                   //RegPayloadLength, 21bytes for length & Fixed length
  0x3C95,                   //RegFiFoThresh   
  
  //0x1888,                   //RegLNA, 200R  
  0x1808,					//RegLNA,50R. (LnaZin=0,50R;=1,200R)
  //0x581B,                   //RegTestLna, Normal sensitivity
  0x582D,                   //RegTestLna, increase sensitivity with LNA (Note: consumption also increase!)
  0x6F30,                   //RegTestDAGC, Improved DAGC
  //0x6F00,                   //RegTestDAGC, Normal DAGC
  0x0104,                   //Enter standby mode    
};

code unsigned int MD1239RxTbl[6] = 
{       
  0x2603,		    //DIO5 clkout, clkout=FOSC/8(32M/8=4MHZ)	
  0x2544,                   //DIO Mapping for Rx
  0x1310,                   //OCP enabled
  0x5A55,                   //Normal and Rx
  0x5C70,                   //Normal and Rx   
  0x0110,                    //Enter Rx mode
};

code unsigned int MD1239RxTb2[7] = 
{       
 

  0x0260,                   //RegDataModul, FSK continous mode without syn word  
   0x2603,		    		//DIO5 clkout, clkout=FOSC/8(32M/8=4MHZ)	
  0x2500,                   //DIO Mapping for Rx(DIO2 as directly out RX RAW data(wave))
  0x1310,                   //OCP enabled
  0x5A55,                   //Normal and Rx
  0x5C70,                   //Normal and Rx   
  0x0110                    //Enter Rx mode



};




//=======================RX-----RX=========================

/**********************************************************
**Name:     MD1239_Config
**Function: Initialize MD1239 & set it entry to standby mode
**Input:    none
**Output:   none
**********************************************************/
void MD1239_Config(void)
{
  unsigned char i;
  gb_RateBuf_Addr=1;
  gb_FreqBuf_Addr=2;			//868mhz
 
  for(i=0;i<3;i++)                      
    SPIWrite(MD1239FreqTbl[gb_FreqBuf_Addr][i]);             //setting frequency parameter
  for(i=0;i<2;i++)
    SPIWrite(MD1239RateTbl[gb_RateBuf_Addr][i]);             //setting rf rate parameter
 
   for(i=0;i<18;i++)
   SPIWrite(MD1239ConfigTbl[i]);							 //normal working rate
}


/**********************************************************
**Name:     MD1239_EntryRx
**Function: Set MD1239 entry Rx_mode
**Input:    None
**Output:   "0" for Error Status
**********************************************************/
void MD1239_EntryRx(void)
{
  unsigned char i; 
  LNA_EN=1;
  MD1239_Config();                                          //Module parameter setting
  
  for(i=0;i<6;i++)                                         //Define to Rx mode  
    SPIWrite(MD1239RxTbl[i]);
 }

/**********************************************************
**Name:     MD1239_RxWaitStable
**Function: Determine whether the state of stable Rx
**Input:    none
**Output:   none
**********************************************************/
/*void MD1239_RxWaitStable(void)
{ 
  u8 temp=0;
  if(gb_WaitStableFlag==0x0f) 
  {
    if(gb_SysTimer_1S!=0)
    {
      temp=SPIRead(0x27);
      if((temp&0xC0)==0xC0 && temp!=0xff)
      {
        gb_WaitStableFlag=1;        
      }
    }
    else
    {
      gb_ErrorFlag=1;
      gb_WaitStableFlag=0;
    }
  }
}
*/
/**********************************************************
**Name:     MD1239_ClearFIFO
**Function: Change to RxMode from StandbyMode, can clear FIFO buffer
**Input:    None
**Output:   None
**********************************************************/
void MD1239_ClearFIFO(void)
{
  SPIWrite(0x0104);                                        //Standby
  SPIWrite(0x0110);                                        //entry RxMode
}

/**********************************************************
**Name:     MD1239_Sleep
**Function: Set MD1239 to sleep mode 
**Input:    none
**Output:   none
**********************************************************/
void MD1239_Sleep(void)
{
  SPIWrite(0x0100);                                        //Sleep
  LNA_EN=0;
}

/**********************************************************
**Name:     MD1239_Standby
**Function: Set MD1239 to Standby mode
**Input:    none
**Output:   none
**********************************************************/
/*void MD1239_Standby(void)
{
  SPIWrite(0x0104);                                        //Standby
  LNA_EN=0;
}
*/
/**********************************************************
**Name:     MD1239_ReadRSSI
**Function: Read the RSSI value
**Input:    none
**Output:   temp, RSSI value
**********************************************************/
/*u8 MD1239_ReadRSSI(void)
{
  u16 temp=0xff;
   //SPIWrite(0x2301);  		//start RSSI measure
   //SPIWrite(0x2301);
  //Delayn(300);
  //if((SPIRead(0x23)&0x02)==0x02)
  if((SPIRead(0x24)&0x02)==0x02)
  {
    temp=SPIRead(0x24);
    temp=0xff-temp;
    temp>>=1;
    temp&=0xff;
  }
  Delayn(500);
  return (u8)temp;
}
*/


/**********************************************************
**Name:     MD1239_RxPacket
**Function: Check for receive one packet
**Input:    none
**Output:   "!0"-----Receive one packet
**          "0"------Nothing for receive
**********************************************************/
void MD1239_RxPacket(void)
{
  unsigned char i; 
 
  if(DIO0==1)
  {
    	for(i=0;i<6;i++) 
      	gb_RxData[i] = 0x00;
      
    	SPIBurstRead(0x00, gb_RxData, 6);  
    	MD1239_ClearFIFO();
 
			SendOut(0x88);
			SendOut(gb_RxData[0]);  //0x03
			SendOut(gb_RxData[1]);	//IDH 0x34
            		SendOut(gb_RxData[2]);	//IDL 0x56
			SendOut(gb_RxData[3]);	//   0x32
			SendOut(gb_RxData[4]);	//   0x03
			SendOut(gb_RxData[5]);	//CRC

		 //compare CRC
		 gb_wr2=0xff;
		 rf_crc_8(gb_RxData[0]);   //type|mode
		 rf_crc_8(gb_RxData[1]);   //IDH
		 rf_crc_8(gb_RxData[2]);   //IDL
		 rf_crc_8(gb_RxData[3]);   //level
		 rf_crc_8(gb_RxData[4]);   //TX remote on,off command, =0x00 no,=0x11 on or off command
		 SendOut(gb_wr2);	//CRC

		 if(gb_wr2==gb_RxData[5])	 //CRC if equal
		 {
                         
                         SendOut(0x66);
			 Light_LED();
                         gb_led_on_time=50;
			 
			 //key_tone();
                         //Delayn(200);

                 
		}
		
	}
}

/**********************************************************
*turn on led according to bat status
**********************************************************/
void Light_LED(void)
{
//				gb_bat_status=c_low_R;
				RLED=1;
				GLED=0;
}
/**********************************************************
**Parameter table define in test mode
**********************************************************/


/**********************************************************
**Name:     MD1239_EntryTestRx
**Function: Set MD1239 entry Rx test mode
**Input:    None
**Output:   None
**********************************************************/
void MD1239_EntryTestRx(void)
{
  unsigned char i;
   LNA_EN=1;
  gb_FreqBuf_Addr=2;
  
  gb_RateBuf_Addr=0;		//0.6kbps
  MD1239_Config();                                          //Module parameter setting
  for(i=0;i<7;i++)                                         //Define to Rx mode  
    SPIWrite(MD1239RxTb2[i]);
  
 
}

/**********************************************************
**Name:     MD1239_TestRx
**Function: MD1239 Rx test mode
**Input:    None
**Output:   "0" for Error Status
**********************************************************/
/*void MD1239_TestRx(void)
{
  //MD1239_RxWaitStable();
  if(gb_WaitStableFlag==1)
  {
    gb_WaitStableFlag=2;
    gb_StatusRx=1;    
  }
}
  */

//=======================================
 unsigned char rf_crc_8(unsigned char temp)
{
	u8 i;
	gb_wr2^=temp;		//xor
	for(i=0;i<8;i++)
	{
	 if((gb_wr2&0x01)!=0)
	 	{gb_wr2>>=1;
		 gb_wr2&=0x7f;
	 	gb_wr2^=0x8c;
	      }
	 else
	 {
	 gb_wr2>>=1;
	 gb_wr2&=0x7f;
	 
	 }
	 
	 		
	}
	return gb_wr2;
}
/**********************************************************
/**********************************************************
**Parameter table define in test mode
**********************************************************/
code u16 MD1239TestConfigTbl[12] = 
{ 
  0x0248,                   //RegDataModul, OOK
  0x0502,                   //RegFdevMsb, 241*61Hz = 35KHz  
  0x0641,                   //RegFdevLsb
  0x1952,                   //RegRxBw , RxBW, 83KHz
  //0x1888,                   //RegLNA, 200R  
  0x1808,		    //50R
  0x2C00,                   //RegPreambleMsb  
  0x2D00,                   //RegPreambleLsb, 0Byte Preamble
  0x2E00,                   //Disable Sync
  0x3700,                   //RegPacketConfig1, Disable CRCNRZ encode
  0x581B,                   //RegTestLna, Normal sensitivity
  0x6F30,                   //RegTestDAGC
  0x0104                    //Enter standby mode  
};

/**********************************************************
**Name:     MD1239_TestConfig
**Function: Initialize in test mode & set it entry to standby mode
**Input:    none
**Output:   none
**********************************************************/
void MD1239_TestConfig(void)
{
  u8 i;
  for(i=0;i<3;i++)                      
    SPIWrite(MD1239FreqTbl[gb_FreqBuf_Addr][i]);           //setting frequency parameter
  for(i=0;i<2;i++)
    SPIWrite(MD1239RateTbl[gb_RateBuf_Addr][i]);           //setting rf rate parameter
  for(i=0;i<12;i++)
    SPIWrite(MD1239TestConfigTbl[i]);                      //setting base parameter
}